function [comod, meanAmp] = f_comodulogram(rawData, sampRate, phaseFreq, ampFreq)
%Comodulogram analysis to assess phase-amplitude cross-frequency coupling
%  over a wide range of frequency bands using Modulation Index (MI)
%
%  Inputs:
%       rawData: one-dimensional raw signal (1xn or nx1)
%       sampRate: sampling rate of raw signal
%       phaseFreq: cutoff frequencies for phase frequency bands
%       ampFreq: cutoff frequencies for amplitude frequency bands
%  Outputs:
%       comod: matrix with MI values for all low/high frequency band pairs
%       meanAmp: Average power of high frequencies in each phase-bin
%
%  Mojtaba Bandarabadi, Inselspital, Bern, April, 2015
%  mojtaba.bandarabadi@unil.ch
%

%%
if size(rawData,1)>size(rawData,2)
    rawData = rawData';
end

%% Preallocation
nPhase = length(phaseFreq);
nAmp = length(ampFreq);
Phase = zeros(nPhase, length(rawData));
Amp   = zeros(nAmp, length(rawData));
comod = zeros(nPhase, nAmp);

%% Extracting envelope for amplitude frequency bands
for i = 1:nAmp
    order = round(3*(sampRate/ampFreq(i,1)));
    fir1Coef = fir1(order,[ampFreq(i,1),ampFreq(i,2)]./(sampRate/2));
    ampSignal = filtfilt(fir1Coef, 1, rawData);
    Amp(i,:) = abs(hilbert(ampSignal));
end

%% Extracting phase for phase frequency bands
for i = 1:nPhase
    order = round(3*(sampRate/phaseFreq(i,1)));
    fir1Coef = fir1(order,[phaseFreq(i,1),phaseFreq(i,2)]./(sampRate/2));
    phaseSignal = filtfilt(fir1Coef, 1, rawData);
    Phase(i,:) = angle(hilbert(phaseSignal));
end

%% Dividing phase [0 2*pi] into several equal bins
nBin = 18; % 18 bins, each bin covers 20 degrees
binStart = zeros(1, nBin);
binSize = 2*pi/nBin;
for i = 1:nBin
    binStart(i) = (i-1)*binSize-pi; % start of each bin
end

%% Calculating Kullback-Leibler (KL) distance
Amp = Amp';
meanAmp = zeros(nPhase, nBin, nAmp);
for i = 1:nPhase
    meanAmpTemp = zeros(nBin, nAmp);
    for k = 1:nBin
        meanAmpTemp(k,:) = nanmean(Amp(Phase(i,:)>=binStart(k) & Phase(i,:)<binStart(k)+binSize, :));
    end
    meanAmpTemp = meanAmpTemp./repmat(sum(meanAmpTemp),size(meanAmpTemp,1),1); % normalize phase-amplitude distribution
    meanAmp(i,:,:) = meanAmpTemp;
    KLdist = meanAmpTemp.*log(nBin.*meanAmpTemp); % Calculate KL distance
    KLdist(isnan(KLdist)) = 0;
    comod(i,:) = sum(KLdist)./log(nBin); % Calculate Modulation Index
end

end
